@@ -95,7 +95,11 @@ speech_build <- function(file, add.error.sir = NULL, rm.error.leg = NULL, compil
                                      "date",
                                      "id"))
         } else {
-            invisible(add_sex(clean_t(out)))
+            invisible(
+                    SError(
+                            add_sex(
+                                    clean_t(
+                                            out))))
         }
 }
 
